# -*- coding: utf-8 -*-

import mock
import os
import pytest

from passerelle.apps.maelis.models import Maelis, Link

from passerelle.utils.jsonresponse import APIError

import utils

pytestmark = pytest.mark.django_db


def get_xml_file(filename):
    return open(os.path.join(os.path.dirname(__file__),
                             'data', 'maelis', filename), 'rb').read()


@pytest.fixture
def family_service_wsdl():
    return get_xml_file('FamilyService.wsdl')


@pytest.fixture
def invoice_service_wsdl():
    return get_xml_file('InvoiceService.wsdl')


@pytest.fixture
def activity_service_wsdl():
    return get_xml_file('ActivityService.wsdl')


@pytest.fixture
def connector(db):
    return utils.setup_access_rights(Maelis.objects.create(
        slug='test', login='foo', password='bar',
        base_url='http://www3.sigec.fr/entrouvertws/services/'))


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_link_unlink(mocked_post, mocked_get, family_service_wsdl,
                     connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamilyByPasswordError.xml'),
                                         status_code=500,
                                         headers={'Content-Type': 'text/xml'})
    assert Link.objects.count() == 0
    resp = app.post_json('/maelis/test/link?NameID=local',
                         params={'family_id': '3264', 'password': 'wrong', 'school_year': '2020'})
    assert resp.json['err'] == 1
    assert resp.json['err_desc'] == 'E204 : Le mot de passe est incorrect'
    assert not resp.json['data']
    assert Link.objects.count() == 0

    resp = app.post_json('/maelis/test/unlink?NameID=local')
    assert resp.json['err']
    assert resp.json['err_desc'] == 'User not linked to family'

    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamilyByPasswordResult.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'})
    resp = app.post_json('/maelis/test/link?NameID=local',
                         params={'family_id': '3264', 'password': '123456'})
    assert resp.json['err'] == 0
    assert resp.json['data']
    assert resp.json['data']['number'] == 3264
    assert Link.objects.count() == 1
    link = Link.objects.get()
    assert link.name_id == 'local'
    assert link.family_id == '3264'

    resp = app.post_json('/maelis/test/unlink?NameID=local')
    assert resp.json['err'] == 0
    assert Link.objects.count() == 0


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_family_info(mocked_post, mocked_get, family_service_wsdl,
                     connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readFamily.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'})
    assert Link.objects.count() == 0
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/family-info?NameID=local')
    assert resp.json['data']["number"] == 3264
    assert resp.json['data']['rl1InfoBean']
    assert resp.json['data']['childInfoList']

    resp = app.get('/maelis/test/children-info?NameID=local')
    assert resp.json['data']
    for child in resp.json['data']:
        assert child['id']
        assert child['text']

    resp = app.get('/maelis/test/child-info?NameID=local&childID=21293')
    assert resp.json['data']
    assert resp.json['data']['num'] == '21293'
    assert resp.json['data']['id'] == '21293'


    resp = app.get('/maelis/test/adults-info?NameID=local')
    assert resp.json['data']
    for child in resp.json['data']:
        assert child['id']
        assert child['text']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_activity_list(mocked_post, mocked_get, family_service_wsdl,
                       activity_service_wsdl, connector, app):
    mocked_get.side_effect = (
        utils.FakedResponse(content=family_service_wsdl,
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=activity_service_wsdl,
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    mocked_post.side_effect = (
        utils.FakedResponse(content=get_xml_file('readFamily.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=get_xml_file('readActivityListResponse.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/activity-list?NameID=local&personID=21293')
    assert resp.json['data']
    for activity in resp.json['data']:
        assert activity['id']
        assert activity['text']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_update_coordinates(mocked_post, mocked_get, family_service_wsdl,
                            connector, app):
    mocked_get.return_value = mock.Mock(content=family_service_wsdl)
    mocked_post.side_effect = (
        utils.FakedResponse(content=get_xml_file('updateCoordinatesResponse.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'}),
        utils.FakedResponse(content=get_xml_file('updateCoordinatesError.xml'),
                            status_code=200,
                            headers={'Content-Type': 'text/xml'})
    )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.post_json('/maelis/test/update-coordinates?NameID=local&personID=21293',
                         params={'mail': 'foo@example.com'}
                         )
    assert resp.content is not None

    resp = app.post_json('/maelis/test/update-coordinates?NameID=local&personID=21293',
                         params={'town': 'Paris'}
                         )
    assert resp.json['err']
    assert resp.json['err_desc'] == 'E16 : Le code postal est obligatoire'


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_list_invoices(mocked_post, mocked_get, invoice_service_wsdl,
                       connector, app):
    mocked_get.return_value = mock.Mock(content=invoice_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readInvoicesResponse.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'}
                                         )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/regie/1/invoices?NameID=local')
    assert resp.json['data']
    for invoice in resp.json['data']:
        assert invoice['display_id']
        assert invoice['label']
        assert invoice['total_amount']
        assert not invoice['paid']


@mock.patch('passerelle.utils.Request.get')
@mock.patch('passerelle.utils.Request.post')
def test_get_invoice_details(mocked_post, mocked_get, invoice_service_wsdl,
                             connector, app):
    mocked_get.return_value = mock.Mock(content=invoice_service_wsdl)
    mocked_post.return_value = mock.Mock(content=get_xml_file('readInvoicesResponse.xml'),
                                         status_code=200,
                                         headers={'Content-Type': 'text/xml'}
                                         )
    Link.objects.create(resource=connector, family_id='3264', name_id='local')
    resp = app.get('/maelis/test/regie/1/invoice/3264-53186?NameID=local')
    assert resp.json['data']
    assert resp.json['data']['label'] == 'FACTURATION SEPTEMBRE 2014'
    assert resp.json['data']['display_id'] == '53186'
